use std::sync::{Arc, Mutex};

use rocket::{http::Status, serde::json::Json, Build, Rocket};
use serde::{Deserialize, Serialize};

#[macro_use] extern crate rocket;

#[derive(Debug, Clone, Serialize, Deserialize, Default, PartialEq)]
struct SensorData {
    pm00_1: u32,
    pm02_5: u32,
    pm10_0: u32,
    temp: u32,
    hum: u32,
}

#[get("/sensor")]
async fn sensor_get(state: &rocket::State<Arc<Mutex<SensorData>>>) -> Json<SensorData> {
    Json(state.lock().unwrap().clone())
}

#[put("/sensor", format = "json", data = "<data>")]
fn sensor_put(data: Json<SensorData>, state: &rocket::State<Arc<Mutex<SensorData>>>) -> Status {
    *state.lock().unwrap() = data.0;
    Status::Ok
}

#[launch]
fn rocket() -> Rocket<Build> {
    rocket::build()
        .manage(Arc::new(Mutex::new(SensorData::default())))
        .mount("/", routes![sensor_get, sensor_put])
}

#[cfg(test)]
mod test {
    use rocket::local::blocking::Client;
    use rocket::http::Status;

    use crate::SensorData;

    #[test]
    fn test_gets_and_puts_sensor_data() {
        let client = Client::tracked(super::rocket()).expect("valid rocket instance");
        let response = client.get(uri!(super::sensor_get)).dispatch();
        assert_eq!(response.status(), Status::Ok);
        assert_eq!(response.into_json::<SensorData>().unwrap(), SensorData::default());

        let data = SensorData {
            pm00_1: 1,
            pm02_5: 2,
            pm10_0: 3,
            temp: 4,
            hum: 5,
        };
        let response = client.put(uri!(super::sensor_put)).json(&data).dispatch();
        assert_eq!(response.status(), Status::Ok);
        assert!(response.body().is_none());

        let response = client.get(uri!(super::sensor_get)).dispatch();
        assert_eq!(response.status(), Status::Ok);
        assert_eq!(response.into_json::<SensorData>().unwrap(), data);
    }
}
